@@ -29,8 +29,9 @@
 #include "slave.h"
 #include "misc.h"
 #include "xdmcp.h"
+#include "choose.h"
 
-static const gchar RCSid[]="$Id: display.c,v 1.10 2001/02/13 23:23:57 jirka Exp $";
+static const gchar RCSid[]="$Id: display.c,v 1.15 2001/08/01 19:11:42 baddog Exp $";
 
 /* External vars */
 extern gboolean GdmXdmcp;
@@ -84,7 +85,7 @@ gdm_display_check_loop (GdmDisplay *disp)
   if (disp->retry_count > 4)
     {
       gchar *msg;
-      msg = g_strdup_printf (_("Failed to start X server several times in a short time period; disabling display %s"), disp->name);
+      msg = g_strdup_printf (_("Failed to start the display server several times in a short time period; disabling display %s"), disp->name);
       gdm_error (msg);
       g_free (msg);
       disp->disabled = TRUE;
@@ -100,8 +101,8 @@ gdm_display_check_loop (GdmDisplay *disp)
    */
   if (since_last < 8)
     {
-      gdm_debug ("Will sleep %d seconds before next X server restart attempt",
-                 8 - since_last);
+      gdm_debug ("Will sleep %ld seconds before next X server restart attempt",
+                 (long)(8 - since_last));
       now = time (NULL) + 8 - since_last;
       disp->sleep_before_run = 8 - since_last;
     }
@@ -154,7 +155,6 @@ gdm_display_manage (GdmDisplay *d)
     switch (d->slavepid = fork()) {
 
     case 0:
-
 	setpgid (0, 0);
 
 	/* Close XDMCP fd in slave process */
@@ -181,7 +181,7 @@ gdm_display_manage (GdmDisplay *d)
 	break;
 
     case -1:
-	gdm_error (_("gdm_display_manage: Failed forking gdm slave process for %d"), d->name);
+	gdm_error (_("gdm_display_manage: Failed forking gdm slave process for %s"), d->name);
 
 	return FALSE;
 
@@ -241,7 +241,7 @@ gdm_display_unmanage (GdmDisplay *d)
 void
 gdm_display_dispose (GdmDisplay *d)
 {
-    if (!d)
+    if (d == NULL)
 	return;
 
     if (d->type == TYPE_XDMCP) {
@@ -289,6 +289,10 @@ gdm_display_dispose (GdmDisplay *d)
     g_free (d->bcookie);
     d->bcookie = NULL;
 
+    if (d->indirect_id > 0)
+	    gdm_choose_indirect_dispose_empty_id (d->indirect_id);
+    d->indirect_id = 0;
+
     g_free (d);
 }
 
