@@ -88,6 +88,7 @@
 #define GDM_NEEDPIC    '#' /* need a user picture?, sent after greeter
 			    *  is started */
 #define GDM_READPIC    '%' /* Send a user picture in a temp file */
+#define GDM_ERRBOX     'e' /* Puts string in a popup error box */
 
 /* Different login interruptions */
 #define GDM_INTERRUPT_TIMED_LOGIN 'T'
@@ -203,6 +204,7 @@
 #define GDM_KEY_SHOW_GNOME_CHOOSER "greeter/ShowGnomeChooserSession=true"
 #define GDM_KEY_SHOW_GNOME_FAILSAFE "greeter/ShowGnomeFailsafeSession=true"
 #define GDM_KEY_SHOW_XTERM_FAILSAFE "greeter/ShowXtermFailsafeSession=true"
+#define GDM_KEY_SHOW_LAST_SESSION "greeter/ShowLastSession=true"
 
 #define GDM_SESSION_FAILSAFE_GNOME "GDM_Failsafe.GNOME"
 #define GDM_SESSION_FAILSAFE_XTERM "GDM_Failsafe.XTERM"
@@ -259,7 +261,6 @@ struct _GdmDisplay {
 
 
 typedef struct _GdmIndirectDisplay GdmIndirectDisplay;
-
 struct _GdmIndirectDisplay {
 	int id;
 	struct sockaddr_in* dsp_sa;
@@ -267,6 +268,35 @@ struct _GdmIndirectDisplay {
 	struct in_addr *chosen_host;
 };
 
+/* NOTE: Timeout and max are hardcoded */
+typedef struct _GdmForwardQuery GdmForwardQuery;
+struct _GdmForwardQuery {
+	time_t acctime;
+	struct sockaddr_in* dsp_sa;
+	struct sockaddr_in* from_sa;
+};
+#define GDM_MAX_FORWARD_QUERIES 10
+#define GDM_FORWARD_QUERY_TIMEOUT 30
+
+/* some extra xdmcp opcodes that xdm will happily ignore since they'll be
+ * the wrong xdmcp version anyway */
+#define GDM_XDMCP_PROTOCOL_VERSION 1000
+enum {
+	GDM_XDMCP_FIRST_OPCODE = 1000, /*just a marker, not an opcode */
+
+	GDM_XDMCP_MANAGED_FORWARD = 1000,
+		/* manager -> manager
+		 * A single packet with MANAGED_FORWARD is sent to the
+		 * manager that sent the forward query from the manager to
+		 * which forward query was sent.  Note that there is no
+		 * ack that this was received, thus it could have been
+		 * dropped on the floor, however, it is not completely
+		 * necessary for it to get through.
+		 *
+		 * Argument is ARRAY8 with the hostname of the manager */
+	GDM_XDMCP_LAST_OPCODE /*just a marker, not an opcode */
+};
+
 typedef gboolean (*GSignalFunc) (gint8		signal,
 				 gpointer	data);
 guint	g_signal_add		(gint8		signal,
@@ -298,9 +328,6 @@ void gdm_quit (void);
 
 void gdm_fifo_close (void);
 
-#define gdm_string_empty(x) ((x)==NULL||(x)[0]=='\0')
-#define gdm_sure_string(x) ((x)!=NULL?(x):"")
-
 #endif /* GDM_H */
 
 /* EOF */
