@@ -20,16 +20,20 @@
 #include <gnome.h>
 #include <syslog.h>
 
+#include <vicious.h>
+
 #include "gdm.h"
 #include "misc.h"
 
-static const gchar RCSid[]="$Id: misc.c,v 1.9 2001/01/30 23:23:40 jirka Exp $";
+static const gchar RCSid[]="$Id: misc.c,v 1.15 2001/08/02 16:50:29 hp Exp $";
 
 
 /* Configuration option variables */
 extern gchar *GdmPidFile;
 extern gboolean GdmDebug;
 
+extern char **environ;
+
 
 /**
  * gdm_fail:
@@ -51,7 +55,7 @@ gdm_fail (const gchar *format, ...)
     va_end (args);
 
     /* Log to both syslog and stderr */
-    syslog (LOG_ERR, s);
+    syslog (LOG_CRIT, "%s", s);
     fprintf (stderr, "%s\n", s);
     fflush (stderr);
 
@@ -86,7 +90,7 @@ gdm_info (const gchar *format, ...)
     s = g_strdup_vprintf (format, args);
     va_end (args);
     
-    syslog (LOG_INFO, s);
+    syslog (LOG_INFO, "%s", s);
     
     g_free (s);
 }
@@ -110,7 +114,7 @@ gdm_error (const gchar *format, ...)
     s = g_strdup_vprintf (format, args);
     va_end (args);
     
-    syslog (LOG_ERR, s);
+    syslog (LOG_ERR, "%s", s);
     
     g_free (s);
 }
@@ -141,113 +145,40 @@ gdm_debug (const gchar *format, ...)
     
     /*{ FILE *fp = fopen ("/tmp/foo.gdm", "a"); fprintf (fp, "%s\n", s); fflush (fp); fclose (fp); };*/
     
-    syslog (LOG_ERR, s);	/* FIXME: LOG_DEBUG */
+    syslog (LOG_ERR, "%s", s);	/* FIXME: LOG_DEBUG */
     
     g_free (s);
 }
 
-
-/**
- * gdm_setenv:
- * @var: Variable to set/unset
- * @value: Value to assign to the variable, NULL for unsetting
- *
- * Wrapper around putenv() because POSIX/SUS is incredibly stupid.
- * 
- *   char *foo; char *bar;
- *   putenv (foo); putenv (bar); free (foo); 
- *
- * is legal, while
- *
- *   char *foo;
- *   putenv (foo); free (foo); 
- *
- * isn't.
- *
- * Afterall, providing the programmer with a nice, consistent
- * interface is what the standard C Library is all about. - Duh!
- *
- * Note from George:
- * You cannot free the last env as it could have been something else
- * and could still be in the env!  We just have to leak, there is no
- * recourse.
- * -George
- */
-
-#ifndef HAVE_SETENV
-gint 
-gdm_setenv (const gchar *var, const gchar *value)
+/* clear environment, but keep the i18n ones,
+ * note that this leaks memory so only use before exec */
+void
+gdm_clearenv_no_lang (void)
 {
-#if 0
-    static gchar *lastenv = NULL; /* Holds last successful assignment pointer */
-#endif
-    gchar *envstr;		  /* Temporary environment string */
-    gint result;		  /* Return value from the putenv() call */
+	int i;
+	GList *li, *envs = NULL;
 
-    /* `var' is a prerequisite */
-    if (!var)
-	return -1;
+	for (i = 0; environ[i] != NULL; i++) {
+		char *env = environ[i];
+		if (strncmp (env, "LC_", 3) == 0 ||
+		    strncmp (env, "LANG", 4) == 0 ||
+		    strncmp (env, "LINGUAS", 7) == 0)
+			envs = g_list_prepend (envs, g_strdup (env));
+	}
 
-    /* `value' is a prerequisite */
-    if (!value)
-	return -1;
+	ve_clearenv ();
 
-    envstr = g_strconcat (var, "=", value, NULL);
+	for (li = envs; li != NULL; li = li->next) {
+		putenv (li->data);
+	}
 
-    /* If string space allocation failed then abort */
-    if (!envstr)
-	return -1;
-
-    /* Stuff the resulting string into the environment */
-    result = putenv (envstr);
-
-#if 0
-    /* If putenv() succeeded and lastenv is set, free the old pointer */
-    if (result == 0 && lastenv)
-	g_free (lastenv);
-
-    /* Save the current string pointer for the next gdm_setenv call */
-    lastenv = envstr;
-#endif
-    
-    return result;
+	g_list_free (envs);
 }
-#endif
-
-#ifndef HAVE_UNSETENV
-gint 
-gdm_unsetenv (const gchar *var)
-{
-#if 0
-    static gchar *lastenv = NULL; /* Holds last successful assignment pointer */
-#endif
-    gchar *envstr;		  /* Temporary environment string */
-    gint result;		  /* Return value from the putenv() call */
-
-    /* `var' is a prerequisite */
-    if (!var)
-	return -1;
-
-    envstr = g_strdup (var);
-
-    /* If string space allocation failed then abort */
-    if (!envstr)
-	return -1;
-
-    /* Stuff the resulting string into the environment */
-    result = putenv (envstr);
-
-#if 0
-    /* If putenv() succeeded and lastenv is set, free the old pointer */
-    if (result == 0 && lastenv)
-	g_free (lastenv);
-
-    /* Save the current string pointer for the next gdm_setenv call */
-    lastenv = envstr;
-#endif
-    
-    return result;
-}
-#endif
 
 /* EOF */
+
+
+
+
+
+
