@@ -31,6 +31,8 @@
 #include <errno.h>
 #include <X11/Xlib.h>
 
+#include <vicious.h>
+
 #include "gdm.h"
 #include "server.h"
 #include "misc.h"
@@ -38,7 +40,7 @@
 #include "display.h"
 #include "auth.h"
 
-static const gchar RCSid[]="$Id: server.c,v 1.26 2001/07/06 08:53:56 jirka Exp $";
+static const gchar RCSid[]="$Id: server.c,v 1.28 2001/08/02 16:50:29 hp Exp $";
 
 #define SERVER_WAIT_ALARM 10
 
@@ -193,7 +195,7 @@ display_busy (GdmDisplay *disp)
 		 * of course additions are welcome to make this more robust */
 		if (strstr (buf, "Server is already active for display")
 		    != NULL) {
-			gdm_error (_("Display %d is busy, there is another "
+			gdm_error (_("Display %s is busy, there is another "
 				     "X server already running"),
 				   disp->name);
 			fclose (fp);
@@ -233,7 +235,7 @@ gdm_server_start (GdmDisplay *disp)
     /* Create new cookie */
     if ( ! gdm_auth_secure_display (d))
 	    return FALSE;
-    gdm_setenv ("DISPLAY", d->name);
+    ve_setenv ("DISPLAY", d->name, TRUE);
 
     /* Catch USR1 from X server */
     usr1.sa_handler = gdm_server_usr1_handler;
@@ -365,7 +367,7 @@ gdm_server_spawn (GdmDisplay *d)
     int len, i;
 
     if (d == NULL ||
-	gdm_string_empty (d->command)) {
+	ve_string_empty (d->command)) {
 	    return;
     }
 
@@ -442,7 +444,7 @@ gdm_server_spawn (GdmDisplay *d)
 	sigaddset (&mask, SIGTERM);
 	sigprocmask (SIG_UNBLOCK, &mask, NULL);
 
-	argv = gdm_split (d->command);
+	argv = ve_split (d->command);
 	for (len = 0; argv != NULL && argv[len] != NULL; len++)
 		;
 
