@@ -25,12 +25,14 @@
   #include <crypt.h>
 #endif /* HAVE_CRYPT */
 
+#include <vicious.h>
+
 #include "gdm.h"
 #include "misc.h"
 #include "slave.h"
 #include "verify.h"
 
-static const gchar RCSid[]="$Id: verify-crypt.c,v 1.10 2001/05/13 06:44:29 jirka Exp $";
+static const gchar RCSid[]="$Id: verify-crypt.c,v 1.14 2001/08/10 19:14:18 jirka Exp $";
 
 
 /* Configuration option variables */
@@ -56,7 +58,7 @@ gdm_verify_user (const char *username,
 		 const gchar *display,
 		 gboolean local) 
 {
-    gchar *login, *passwd, *ppasswd;
+    gchar *login, *passwd, *ppasswd, *auth_errmsg;
     struct passwd *pwent;
 
     if (local)
@@ -64,7 +66,7 @@ gdm_verify_user (const char *username,
 
     /* Ask for the user's login */
     if (username == NULL) {
-	    login = gdm_slave_greeter_ctl (GDM_LOGIN, _("Login:"));
+	    login = gdm_slave_greeter_ctl (GDM_LOGIN, _("Username:"));
 	    if (login == NULL ||
 		gdm_slave_greeter_check_interruption (login)) {
 		    if (local)
@@ -78,10 +80,11 @@ gdm_verify_user (const char *username,
 
     pwent = getpwnam (login);
         
-    ppasswd = !pwent ? NULL : pwent->pw_passwd;
+    ppasswd = (pwent == NULL) ? NULL : g_strdup (pwent->pw_passwd);
     
     /* Request the user's password */
-    if (gdm_string_empty (ppasswd)) {
+    if (pwent != NULL &&
+        ve_string_empty (ppasswd)) {
 	    /* eeek a passwordless account */
 	    passwd = g_strdup ("");
     } else {
@@ -93,28 +96,50 @@ gdm_verify_user (const char *username,
 			    gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
 		    g_free (login);
 		    g_free (passwd);
+		    g_free (ppasswd);
 		    return NULL;
-    }
-
+	    }
     }
 
     if (local)
 	    gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
 
     if (pwent == NULL) {
-	    gdm_error (_("Couldn't authenticate %s"), login);
-	    gdm_slave_greeter_ctl_no_ret (GDM_MSGERR, _("Login incorrect"));
+	    gdm_error (_("Couldn't authenticate user"));
+	    if (GdmVerboseAuth) {
+		    auth_errmsg = g_strdup_printf
+			    (_("\nIncorrect username or password.  "
+			       "Letters must be typed in the correct case.  "  
+			       "Please be sure the Caps Lock key is not enabled"));
+		    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX, auth_errmsg);
+		    g_free (auth_errmsg);
+	    } else {
+		    gdm_slave_greeter_ctl_no_ret (GDM_MSGERR,
+						  _("Login incorrect"));
+	    }
 	    g_free (login);
 	    g_free (passwd);
+	    g_free (ppasswd);
 	    return NULL;
     }
 
     /* Check whether password is valid */
     if (ppasswd == NULL || (ppasswd[0] != '\0' &&
 			    strcmp (crypt (passwd, ppasswd), ppasswd) != 0)) {
-	    gdm_slave_greeter_ctl_no_ret (GDM_MSGERR, _("Login incorrect"));
+	    if (GdmVerboseAuth) {
+		    auth_errmsg = g_strdup_printf
+			    (_("\nIncorrect username or password.  "
+			       "Letters must be typed in the correct case.  "  
+			       "Please be sure the Caps Lock key is not enabled"));
+		    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX, auth_errmsg);
+		    g_free (auth_errmsg);
+	    } else {
+		    gdm_slave_greeter_ctl_no_ret (GDM_MSGERR,
+						  _("Login incorrect"));
+	    }
 	    g_free (login);
 	    g_free (passwd);
+	    g_free (ppasswd);
 	    return NULL;
     }
 
@@ -122,15 +147,20 @@ gdm_verify_user (const char *username,
 	  ( ! GdmAllowRemoteRoot && ! local) ) &&
 	pwent->pw_uid == 0) {
 	    gdm_error (_("Root login disallowed on display '%s'"), display);
-	    if (GdmVerboseAuth) {
-		    gdm_slave_greeter_ctl_no_ret (GDM_MSGERR,
-						  _("Root login disallowed"));
-	    } else {
-		    gdm_slave_greeter_ctl_no_ret (GDM_MSGERR,
+  	    if (GdmVerboseAuth) {
+ 		    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX,
+						  _("The system administrator "
+						    "is not allowed to login "
+						    "from this screen"));
+ 		    /*gdm_slave_greeter_ctl_no_ret (GDM_MSGERR,
+ 		      _("Root login disallowed"));*/
+  	    } else {
+ 		    gdm_slave_greeter_ctl_no_ret (GDM_MSGERR,
 						  _("Login incorrect"));
-	    }
+  	    }
 	    g_free (login);
 	    g_free (passwd);
+	    g_free (ppasswd);
 	    return NULL;
     }
 
@@ -139,16 +169,24 @@ gdm_verify_user (const char *username,
 	strcmp (pwent->pw_shell, "/bin/false") == 0) {
 	    gdm_error (_("User %s not allowed to log in"), login);
 	    if (GdmVerboseAuth) {
-		    gdm_slave_greeter_ctl_no_ret (GDM_MSGERR,
-						  _("Login disabled"));
+		    gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX,
+						  _("\nThe system administrator"
+						    " has disabled your "
+						    "account."));
+		    /*gdm_slave_greeter_ctl_no_ret (GDM_MSGERR,
+		      _("Login disabled"));*/
 	    } else {
 		    gdm_slave_greeter_ctl_no_ret (GDM_MSGERR,
 						  _("Login incorrect"));
 	    }
+	    g_free (login);
+	    g_free (passwd);
+	    g_free (ppasswd);
 	    return NULL;
     }	
 
     g_free (passwd);
+    g_free (ppasswd);
     
     return login;
 }
