@@ -44,12 +44,14 @@
 #include <sys/ioctl.h>
 #include <net/if.h>
 
+#include <viciousui.h>
+
 #include "gdmchooser.h"
 #include "gdm.h"
 #include "misc.h"
 #include "gdmwm.h"
 
-static const gchar RCSid[]="$Id: gdmchooser.c,v 1.26 2001/07/05 10:27:30 jirka Exp $";
+static const gchar RCSid[]="$Id: gdmchooser.c,v 1.28 2001/08/04 23:28:52 jirka Exp $";
 
 /* set the DOING_GDM_DEVELOPMENT env variable if you want to
  * search for the glade file in the current dir and not the system
@@ -473,7 +475,7 @@ gdm_chooser_abort (const gchar *format, ...)
     s = g_strdup_vprintf (format, args);
     va_end (args);
 
-    syslog (LOG_ERR, s);
+    syslog (LOG_ERR, "%s", s);
     closelog ();
 
     exit (EXIT_FAILURE);
@@ -507,7 +509,7 @@ gdm_chooser_parse_config (void)
     GdmBroadcast = gnome_config_get_bool (GDM_KEY_BROADCAST);
     /* if broadcasting, then append BROADCAST to hosts */
     if (GdmBroadcast) {
-	    if (gdm_string_empty (GdmHosts)) {
+	    if (ve_string_empty (GdmHosts)) {
 		    g_free (GdmHosts);
 		    GdmHosts = "BROADCAST";
 	    } else {
@@ -980,7 +982,7 @@ set_background (void)
 		GdkColor color;
 		GdkColormap *colormap;
 
-		if (gdm_string_empty (GdmBackgroundColor) ||
+		if (ve_string_empty (GdmBackgroundColor) ||
 		    ! gdk_color_parse (GdmBackgroundColor, &color)) {
 			gdk_color_parse ("#007777", &color);
 		}
