@@ -21,12 +21,16 @@
  */
 
 #include <config.h>
+#include <gnome.h>
+
 #include <errno.h>
 #include <dirent.h>
 #include <sys/stat.h>
 #include <syslog.h>
+
+#include <viciousui.h>
+
 #include "gdmconfig.h"
-#include "icon-entry-hack.h"
 #include "misc.h"
 
 /* set the DOING_GDM_DEVELOPMENT env variable if you don't
@@ -165,13 +169,13 @@ gdm_event (GtkObject *object,
 static void
 check_binary (GtkEntry *entry)
 {
-	char *bin = gdm_first_word (gtk_entry_get_text (entry));
+	char *bin = ve_first_word (gtk_entry_get_text (entry));
 
-	if ( ! gdm_string_empty (bin) &&
+	if ( ! ve_string_empty (bin) &&
 	    access (bin, X_OK) == 0)
-		entry_set_red (GTK_WIDGET (entry), FALSE);
+		ve_entry_set_red (GTK_WIDGET (entry), FALSE);
 	else
-		entry_set_red (GTK_WIDGET (entry), TRUE);
+		ve_entry_set_red (GTK_WIDGET (entry), TRUE);
 
 	g_free (bin);
 }
@@ -182,17 +186,17 @@ check_dir (GtkEntry *entry)
 	char *text = gtk_entry_get_text (entry);
 
 	/* first try access as it's a LOT faster then stat */
-	if ( ! gdm_string_empty (text) &&
+	if ( ! ve_string_empty (text) &&
 	    access (text, R_OK) == 0) {
 		struct stat sbuf;
 		/* check for this being a directory */
 		if (stat (text, &sbuf) < 0 ||
 		    ! S_ISDIR (sbuf.st_mode))
-			entry_set_red (GTK_WIDGET (entry), TRUE);
+			ve_entry_set_red (GTK_WIDGET (entry), TRUE);
 		else
-			entry_set_red (GTK_WIDGET (entry), FALSE);
+			ve_entry_set_red (GTK_WIDGET (entry), FALSE);
 	} else {
-		entry_set_red (GTK_WIDGET (entry), TRUE);
+		ve_entry_set_red (GTK_WIDGET (entry), TRUE);
 	}
 }
 
@@ -209,17 +213,17 @@ check_dirname (GtkEntry *entry)
 		return;
 
 	/* first try access as it's a LOT faster then stat */
-	if ( ! gdm_string_empty (dir) &&
+	if ( ! ve_string_empty (dir) &&
 	    access (dir, R_OK) == 0) {
 		struct stat sbuf;
 		/* check for this being a directory */
 		if (stat (dir, &sbuf) < 0 ||
 		    ! S_ISDIR (sbuf.st_mode))
-			entry_set_red (GTK_WIDGET (entry), TRUE);
+			ve_entry_set_red (GTK_WIDGET (entry), TRUE);
 		else
-			entry_set_red (GTK_WIDGET (entry), FALSE);
+			ve_entry_set_red (GTK_WIDGET (entry), FALSE);
 	} else {
-		entry_set_red (GTK_WIDGET (entry), TRUE);
+		ve_entry_set_red (GTK_WIDGET (entry), TRUE);
 	}
 
 	g_free (dir);
@@ -230,11 +234,11 @@ check_file (GtkEntry *entry)
 {
 	char *text = gtk_entry_get_text (entry);
 
-	if ( ! gdm_string_empty (text) &&
+	if ( ! ve_string_empty (text) &&
 	    access (text, R_OK) == 0) {
-		entry_set_red (GTK_WIDGET (entry), FALSE);
+		ve_entry_set_red (GTK_WIDGET (entry), FALSE);
 	} else {
-		entry_set_red (GTK_WIDGET (entry), TRUE);
+		ve_entry_set_red (GTK_WIDGET (entry), TRUE);
 	}
 }
 
@@ -335,7 +339,7 @@ main (int argc, char *argv[])
 		gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
 		gtkrc = gnome_config_get_string (GDM_KEY_GTKRC);
 		gnome_config_pop_prefix ();
-		if ( ! gdm_string_empty (gtkrc))
+		if ( ! ve_string_empty (gtkrc))
 			gtk_rc_parse (gtkrc);
 		g_free (gtkrc);
 	}
@@ -710,7 +714,7 @@ gdm_config_parse_most (gboolean factory)
 
         /* If default session link exists, find out what it points to */
 	if (S_ISLNK (statbuf.st_mode) &&
-	    strcasecmp_no_locale (dent->d_name, "default") == 0) 
+	    ve_strcasecmp_no_locale (dent->d_name, "default") == 0) 
 	 {
 	    gchar t[_POSIX_PATH_MAX];
 	    
@@ -721,7 +725,7 @@ gdm_config_parse_most (gboolean factory)
 	    * things up.
 	    */
 	   if (strcmp (g_basename (s), t) != 0) {
-		   printf ("recording %s as default session.\n", t);
+		   /* printf ("recording %s as default session.\n", t); */
 		   g_free (default_session_link_name);
 		   default_session_link_name = g_strdup (dent->d_name);
 		   g_free (default_session_name);
@@ -734,8 +738,7 @@ gdm_config_parse_most (gboolean factory)
 
 	    if ((statbuf.st_mode & (S_IRUSR|S_IXUSR)) == (S_IRUSR|S_IXUSR) &&
 		(statbuf.st_mode & (S_IRGRP|S_IXGRP)) == (S_IRGRP|S_IXGRP) &&
-		(statbuf.st_mode & (S_IROTH|S_IXOTH)) == (S_IROTH|S_IXOTH)) 
-	    {
+		(statbuf.st_mode & (S_IROTH|S_IXOTH)) == (S_IROTH|S_IXOTH)) {
 	       gchar *row[1];
 	       int rowNum;
 	       GdmConfigSession *this_session;
@@ -744,14 +747,17 @@ gdm_config_parse_most (gboolean factory)
 	       this_session = g_malloc0 (sizeof (GdmConfigSession));
 	       this_session->name = g_strdup(dent->d_name);
 	       script_file = fopen(s, "r");
-	       if (!script_file) {
-		  gnome_error_dialog_parented (_("Error reading session script!"),
-					       GTK_WINDOW (GDMconfigurator));
-		  this_session->script_contents = g_strdup (_("Error reading this session script"));
-		  this_session->changable = FALSE;
+	       if (script_file == NULL) {
+		       gnome_error_dialog_parented
+			       (_("Error reading session script!"),
+				GTK_WINDOW (GDMconfigurator));
+		       this_session->script_contents =
+			       g_strdup (_("Error reading this session script"));
+		       this_session->changable = FALSE;
 	       } else {
 		       GString *str = g_string_new (NULL);
 		       gchar buffer[BUFSIZ];
+
 		       while (fgets (buffer, sizeof (buffer),
 				     script_file) != NULL) {
 			       g_string_append (str, buffer);
@@ -760,8 +766,8 @@ gdm_config_parse_most (gboolean factory)
 		       g_string_free (str, FALSE);
 		       /* printf ("got script contents:\n%s.\n", this_session->script_contents); */
 		       this_session->changable = TRUE;
+		       fclose (script_file);
 	       }
-	       fclose (script_file);
 	       this_session->changed = FALSE;
 	       this_session->is_default = FALSE;
 
@@ -773,11 +779,11 @@ gdm_config_parse_most (gboolean factory)
 				       rowNum,
 				       this_session);
 
-	    }
-	    else 
+	    } else  {
 	        /* FIXME: this should have an error dialog I suppose */
 		syslog (LOG_ERR, "Wrong permissions on %s/%s. Should be readable/executable for all.", 
 			sessions_directory, dent->d_name);
+	    }
 	}
 
 	dent = readdir (sessdir);
@@ -794,7 +800,7 @@ gdm_config_parse_most (gboolean factory)
 	 g_assert (data != NULL);
 	 if (strcmp(default_session_name, data->name) == 0) {
 	    
-	    printf ("coloring session %s.\n", data->name);
+	    /*printf ("coloring session %s.\n", data->name);*/
 	    gdk_color_parse ("#d6e8ff", &col);
 	    gdk_color_alloc (gdk_colormap_get_system(), &col);
 	    
@@ -876,7 +882,7 @@ gdm_config_parse_remaining (gboolean factory)
     /* Login stuff */
     {
 	    char *login = gnome_config_get_string (GDM_KEY_AUTOMATICLOGIN);
-	    if (gdm_string_empty (login)) {
+	    if (ve_string_empty (login)) {
 		    gdm_toggle_set ("enable_automatic_login", FALSE);
 	    } else {
 		    gdm_toggle_set ("enable_automatic_login", gnome_config_get_bool (GDM_KEY_AUTOMATICLOGIN_ENABLE));
@@ -884,7 +890,7 @@ gdm_config_parse_remaining (gboolean factory)
 	    g_free (login);
 
 	    login = gnome_config_get_string (GDM_KEY_TIMED_LOGIN);
-	    if (gdm_string_empty (login)) {
+	    if (ve_string_empty (login)) {
 		    gdm_toggle_set ("enable_timed_login", FALSE);
 	    } else {
 		    gdm_toggle_set ("enable_timed_login", gnome_config_get_bool (GDM_KEY_TIMED_LOGIN_ENABLE));
@@ -1156,7 +1162,7 @@ write_config (void)
 	full_deleted_file = g_strconcat (sessions_directory, "/", 
 					 file_basename, ".deleted",
 					 NULL);
-	printf ("deleting: %s.\n", full_file);
+	/*printf ("deleting: %s.\n", full_file);*/
 	/* remove old backup file */
 	unlink (full_deleted_file);
 	errno = 0;
@@ -1195,7 +1201,7 @@ write_config (void)
 		char *old;
 		old = g_strconcat (sessions_directory, "/",
 				   session->old_name, NULL);
-		printf ("removing %s\n", session->old_name);
+		/* printf ("removing %s\n", session->old_name); */
 		errno = 0;
 		if (unlink (old) < 0) {
 			if (errors == NULL)
@@ -1219,8 +1225,7 @@ write_config (void)
      }
 
    /* Now write out files that were changed (or renamed above) */
-   for (i=0; i<GTK_CLIST(get_widget ("sessions_clist"))->rows; i++)
-     {
+   for (i=0; i<GTK_CLIST(get_widget ("sessions_clist"))->rows; i++) {
 	GdmConfigSession *session;
 	session = gtk_clist_get_row_data (GTK_CLIST(get_widget ("sessions_clist")),
 					  i);
@@ -1238,7 +1243,7 @@ write_config (void)
 	   else
 		   filename = g_strconcat (sessions_directory, "/",
 					   session->name, NULL);
-	   printf ("writing changes to: %s.\n", filename);
+	   /* printf ("writing changes to: %s.\n", filename); */
 	   errno = 0;
 	   fp = fopen(filename, "w");
 	   if (fp == NULL) {
@@ -1269,8 +1274,7 @@ write_config (void)
 	   /* all went well */
 	   session->changed = FALSE;
 	}
-	
-     }
+   }
 
    if (old_current_default_session != current_default_session) {
 	if (default_session_link_name != NULL) {
@@ -1388,8 +1392,7 @@ revert_to_factory_settings (GtkMenuItem *menu_item,
 }
 
 void
-write_new_config_file                  (GtkButton *button,
-                                        gpointer         user_data)
+write_new_config_file (GtkButton *button, gpointer user_data)
 {
 	write_config ();
 }
@@ -1426,8 +1429,7 @@ open_help_page (GtkButton *button,
 
 
 gint
-exit_configurator                      (void     *gnomedialog,
-                                        gpointer         user_data)
+exit_configurator (GtkWidget *gnomedialog, gpointer user_data)
 {
     gtk_main_quit();
     return 0;
@@ -1710,7 +1712,7 @@ sessions_clist_row_selected                  (GtkCList *clist,
    gtk_editable_set_editable (GTK_EDITABLE (get_widget ("session_name_entry")),
 			      sess_details->changable);
    /* not red by default */
-   entry_set_red (get_widget ("session_name_entry"), FALSE);
+   ve_entry_set_red (get_widget ("session_name_entry"), FALSE);
 
    gtk_signal_handler_unblock_by_func (GTK_OBJECT(get_widget("session_text")),
 				       session_text_edited, NULL);
@@ -1750,7 +1752,7 @@ is_legal_session_name (const char *text, GdmConfigSession *this_session)
 {
 	int i;
 
-	if (gdm_string_empty (text))
+	if (ve_string_empty (text))
 		return FALSE;
 
 	for (i = 0; i < GTK_CLIST(get_widget ("sessions_clist"))->rows; i++) {
@@ -1777,7 +1779,7 @@ add_session_real (gchar *new_session_name, gpointer data)
 	   char *label[1];
 	   int rowNum;
 
-	   printf ("Will add session with %s.\n", new_session_name);
+	   /* printf ("Will add session with %s.\n", new_session_name); */
 	   a_session = g_malloc0 (sizeof (GdmConfigSession));
 	   a_session->name = g_strdup (new_session_name);
 
@@ -1879,7 +1881,7 @@ modify_session_name (GtkEntry *entry, gpointer data)
     * bad UI.  The user probably wiped the entry to typoe in something
     * different*/
    if (is_legal_session_name (text, selected_session)) {
-	entry_set_red (GTK_WIDGET (entry), FALSE);
+	ve_entry_set_red (GTK_WIDGET (entry), FALSE);
 	/* We need to track the first name of this session to rename
 	 * the actual session file.
 	 */
@@ -1893,7 +1895,7 @@ modify_session_name (GtkEntry *entry, gpointer data)
 			    selected_session_row, 0,
 			    selected_session->name);
    } else {
-	entry_set_red (GTK_WIDGET (entry), TRUE);
+	ve_entry_set_red (GTK_WIDGET (entry), TRUE);
    }
 }
 
