@@ -142,25 +142,25 @@ void gdm_radio_write (gchar *radio_base_name,
 void user_level_row_selected(GtkCList *clist, gint row,
 			     gint column, GdkEvent *event, gpointer data);
 void show_about_box(void);
-void 
-gdm_config_parse_most                  (void);
-void 
-gdm_config_parse_remaining             (void);
+void gdm_config_parse_most                  (gboolean factory);
+void gdm_config_parse_remaining             (gboolean factory);
 
 void
 write_new_config_file                  (GtkButton *button,
                                         gpointer         user_data);
 void revert_settings_to_file_state (GtkMenuItem *menu_item,
-									gpointer user_data);
+				    gpointer user_data);
+void revert_to_factory_settings (GtkMenuItem *menu_item,
+				 gpointer user_data);
 void
 write_and_close                        (GtkButton *button,
-										gpointer user_data);
+					gpointer user_data);
 void
 open_help_page                         (GtkButton *button,
                                         gpointer         user_data);
-gint
-exit_configurator                      (void     *gnomedialog,
-                                        gpointer         user_data);
+
+gint exit_configurator (GtkWidget *gnomedialog, gpointer user_data);
+
 void
 can_apply_now                          (GtkEditable     *editable,
                                         gpointer         user_data);
@@ -171,7 +171,7 @@ void
 change_background_sensitivity_image    (GtkButton       *button,
                                         gpointer         user_data);
 void
-change_background_sensitivity_color    (GtkButton       *button,
+change_background_sensitivity_none     (GtkButton       *button,
                                         gpointer         user_data);
 void
 set_face_sensitivity                   (GtkButton       *button,
@@ -228,3 +228,7 @@ void
 modify_session_name (GtkEntry *entry, gpointer data);
 void 
 session_directory_modified (GtkEntry *entry, gpointer data);
+
+void change_automatic_sensitivity (GtkButton *button, gpointer user_data);
+void change_timed_sensitivity (GtkButton *button, gpointer user_data);
+
